// ==========================================================================
// Demo: Remote Update
// Caution: no error check for the sake of briefness and understandability
// Tips:
// (1)  Remote Update Management (RUM) is a utility that may be utilized by ROCKEY developers
//      to implement remote updates to their software protection and licensing systems. RUM is
//      based on "one time use" passwords.
// (2)  The RUM process is described below:( some steps is for developer, others for end user)
//
//      1) "a.Set new basic remote update password"
//         The software developer must first initialize the ROCKEY dongle to set the Basic
//         Remote Update Password (BRUP). This process requires use of the super password. Once
//         the BRUP is set it can only be changed by reformatting the dongle by means of the
//         super password. The initialized dongle may then be distributed to the end user.
//      2) "b.Get remote update information and the current dongle hardware ID"
//         When software developers need to remote update their software, ROCKEY software at
//         the user end gets the relevant information (the current remote update information and
//         hardware ID) and transfers such data by some means to the software developer.
//      3) "d.Change the privilege to super user and acquire the next remote update password."
//         Upon receiving the pertinent information, the software developer gets the next update
//         password for the end user and transfers the new remote update password to the end user.
//      4) "f.return to password not verified status"
//         next step is demo for end user, so return to password not verified status.
//      5) "g.Verify remote update password and check if the remote update password passes the
//          verification."
//         After receiving the new update password from the software developer, the user can send
//         the new password to dongle for verification. If it succeeds, the remote update tag of
//         dongle is set, the program may check the tag status to determine the corresponding
//         actions. Because the dongle internal program can also detect the tag, it is recommended
//         that some part of the update process be performed by the internal program.
// (3) The remote update tag, RemoteTag, is not the update password, it is the information set by
//     the software developer for identifying the current software update status. Usually, we
//     recommend you write the software version here, thus the software developer can tell from
//     which version the user intends to update. It is a 32-bit number, but its highest bit is used
//     to show whether the "hardware ID" is used or not. If the bit is 1, the update password is
//     related to the "Hardware ID" of dongle; if it is 0, it means the update password has nothing
//     to do with the "hardware ID", it only makes reference to the previous update password.
//     "one time password" simplifies update maintenance of the software developer.
// (4) Additionally, the RemoteTag and RemotePass provided to users are related to each other. The
//     user can not modify either of the two. If the highest bit of RemoteTag is 1, the update
//     password is only valid for some specific dongle. In the information the user offers to
//     software developer, only the hardware ID and the previous update password are necessary.
//     The last RemoteTag is merely a reference to the software developer, and it does not affect
//     the next update information. The software developer should set a new RemoteTag as the update
//     mark different from the previous ones for each update operation.
// ==========================================================================

import java.io.*;

public class Sample10 extends JavaRockey
{
	public Sample10()
	{
	}

	public static void main(String [] args)
	{
		int i, num, hic=0, errcode;
		byte []cmddata=new byte[256];	// command buffer
		byte []buffer=new byte[256];	// user buffer
		byte []remotepass=new byte[8]; 
		long RemoteTag;
		long HardSerial;
	
		JavaRockey Rockey=new JavaRockey();
		num = Rockey.DIC_Find();
		for (i=0;i<num;i++)
		{
			hic = Rockey.DIC_Open(i, null);
			if (hic >= 0) break;		// find the first ROCKEY
		}
		if (i == num) return;
	
		// a.Set the new basic remote update password.
		Rockey.DIC_Set(cmddata, Rockey.REMOTE_TAG, Rockey.BY_VALUE, 1, null);	// Set remote tag to 1
		for (i=0;i<8;i++) buffer[i] = 'A';					// set the new remote update password 8 As.
		Rockey.DIC_Set(cmddata, Rockey.REMOTE_PASS, Rockey.BY_ARRAY, 0, buffer);
		errcode = Rockey.DIC_Command(hic, Rockey.SET_REMOTE_INFO, cmddata);	// Update remote update info
	
		// b.Read remote update info
		errcode = Rockey.DIC_Command(hic, Rockey.GET_REMOTE_INFO, cmddata);
		RemoteTag = Rockey.DIC_Get(cmddata, Rockey.REMOTE_TAG, Rockey.BY_VALUE, null);
		Rockey.DIC_Get(cmddata, Rockey.REMOTE_PASS, Rockey.BY_ARRAY, buffer);
		System.out.print("RemoteTag="+RemoteTag+", ");
		System.out.print("RemotePass=");
		for (i=0;i<8;i++)			
			System.out.print(Integer.toHexString(buffer[i]&0xFF)+" ");
			
		System.out.println("");
		for(i=0;i<8;i++)
			remotepass[i]=buffer[i];
	
		// c.Get hardware ID of the current dongle
		errcode = Rockey.DIC_Command(hic, Rockey.GET_HARDWARE_INFO, cmddata);	// Get dongle hardware ID
		HardSerial = Rockey.DIC_Get(cmddata, Rockey.HARD_SERIAL, Rockey.BY_VALUE, null);
	
		// d.Change to super user
		for (i=0;i<8;i++) cmddata[i] = (byte)0xff; //  Super password is 8 FFs.
		errcode = Rockey.DIC_Command(hic, Rockey.CHECK_SUPER_PASS, cmddata);
	
		// e.Get the next update password of the current one
		Rockey.DIC_Set(cmddata, Rockey.UPGRADE_REMOTE_TAG, Rockey.BY_VALUE, 2, null);	//The new update tag is 2
		Rockey.DIC_Set(cmddata, Rockey.UPGRADE_HARD_SERIAL, Rockey.BY_VALUE, (int)HardSerial, null);
		for(i=0;i<8;i++)
			buffer[i]=remotepass[i];
		Rockey.DIC_Set(cmddata, Rockey.UPGRADE_REMOTE_PASS, Rockey.BY_ARRAY, 0, buffer);
		errcode = Rockey.DIC_Command(hic, Rockey.GET_UPGRADE_REMOTE_PASS, cmddata);
		for(i=0;i<8;i++)
			remotepass[i]=cmddata[i];// Transfer the new remote update password to remotepass
	
		// f.Change the super user status to common user status
		errcode = Rockey.DIC_Close(hic);
		errcode = Rockey.DIC_Open(hic, null);
	
		// g.Verify the remote update password
		Rockey.DIC_Set(cmddata, Rockey.REMOTE_TAG, Rockey.BY_VALUE, 2, null);
		for(i=0;i<8;i++)
			buffer[i]=remotepass[i];
		Rockey.DIC_Set(cmddata, Rockey.REMOTE_PASS, Rockey.BY_ARRAY, 0, buffer);
		errcode = Rockey.DIC_Command(hic, Rockey.CHECK_REMOTE_INFO, cmddata);	// Verify remote update info
	
		// h.Check if remote update password verification succeeds
		errcode = Rockey.DIC_Command(hic, Rockey.GET_CARD_PRIVILEGE, cmddata);	// Get privilege info
		if (Rockey.DIC_Get(cmddata, Rockey.REMOTEPASS_STATUS, Rockey.BY_VALUE, null)!=0)
			System.out.println("Remote update password verification succeeded");
		else System.out.println("Remote update password verification failed");
	
		// i.Read the new remote update information
		errcode = Rockey.DIC_Command(hic, Rockey.GET_REMOTE_INFO, cmddata);
		RemoteTag = Rockey.DIC_Get(cmddata, Rockey.REMOTE_TAG, Rockey.BY_VALUE, null);
		Rockey.DIC_Get(cmddata, Rockey.REMOTE_PASS, Rockey.BY_ARRAY, buffer);
		System.out.println("RemoteTag="+RemoteTag+", ");
		System.out.print("RemotePass=");
		for (i=0;i<8;i++) System.out.print(Integer.toHexString(buffer[i]&0XFF)+" ");
	
		errcode = Rockey.DIC_Close(hic);
	}

}